# Generated by Django 2.2.10 on 2020-12-24 13:02

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("drc_cmis", "0012_auto_20200821_1512"),
    ]

    operations = [
        migrations.AlterField(
            model_name="cmisconfig",
            name="time_zone",
            field=models.CharField(
                choices=[
                    ("Africa/Abidjan", "Africa/Abidjan"),
                    ("Africa/Accra", "Africa/Accra"),
                    ("Africa/Addis_Ababa", "Africa/Addis_Ababa"),
                    ("Africa/Algiers", "Africa/Algiers"),
                    ("Africa/Asmara", "Africa/Asmara"),
                    ("Africa/Bamako", "Africa/Bamako"),
                    ("Africa/Bangui", "Africa/Bangui"),
                    ("Africa/Banjul", "Africa/Banjul"),
                    ("Africa/Bissau", "Africa/Bissau"),
                    ("Africa/Blantyre", "Africa/Blantyre"),
                    ("Africa/Brazzaville", "Africa/Brazzaville"),
                    ("Africa/Bujumbura", "Africa/Bujumbura"),
                    ("Africa/Cairo", "Africa/Cairo"),
                    ("Africa/Casablanca", "Africa/Casablanca"),
                    ("Africa/Ceuta", "Africa/Ceuta"),
                    ("Africa/Conakry", "Africa/Conakry"),
                    ("Africa/Dakar", "Africa/Dakar"),
                    ("Africa/Dar_es_Salaam", "Africa/Dar_es_Salaam"),
                    ("Africa/Djibouti", "Africa/Djibouti"),
                    ("Africa/Douala", "Africa/Douala"),
                    ("Africa/El_Aaiun", "Africa/El_Aaiun"),
                    ("Africa/Freetown", "Africa/Freetown"),
                    ("Africa/Gaborone", "Africa/Gaborone"),
                    ("Africa/Harare", "Africa/Harare"),
                    ("Africa/Johannesburg", "Africa/Johannesburg"),
                    ("Africa/Juba", "Africa/Juba"),
                    ("Africa/Kampala", "Africa/Kampala"),
                    ("Africa/Khartoum", "Africa/Khartoum"),
                    ("Africa/Kigali", "Africa/Kigali"),
                    ("Africa/Kinshasa", "Africa/Kinshasa"),
                    ("Africa/Lagos", "Africa/Lagos"),
                    ("Africa/Libreville", "Africa/Libreville"),
                    ("Africa/Lome", "Africa/Lome"),
                    ("Africa/Luanda", "Africa/Luanda"),
                    ("Africa/Lubumbashi", "Africa/Lubumbashi"),
                    ("Africa/Lusaka", "Africa/Lusaka"),
                    ("Africa/Malabo", "Africa/Malabo"),
                    ("Africa/Maputo", "Africa/Maputo"),
                    ("Africa/Maseru", "Africa/Maseru"),
                    ("Africa/Mbabane", "Africa/Mbabane"),
                    ("Africa/Mogadishu", "Africa/Mogadishu"),
                    ("Africa/Monrovia", "Africa/Monrovia"),
                    ("Africa/Nairobi", "Africa/Nairobi"),
                    ("Africa/Ndjamena", "Africa/Ndjamena"),
                    ("Africa/Niamey", "Africa/Niamey"),
                    ("Africa/Nouakchott", "Africa/Nouakchott"),
                    ("Africa/Ouagadougou", "Africa/Ouagadougou"),
                    ("Africa/Porto-Novo", "Africa/Porto-Novo"),
                    ("Africa/Sao_Tome", "Africa/Sao_Tome"),
                    ("Africa/Tripoli", "Africa/Tripoli"),
                    ("Africa/Tunis", "Africa/Tunis"),
                    ("Africa/Windhoek", "Africa/Windhoek"),
                    ("America/Adak", "America/Adak"),
                    ("America/Anchorage", "America/Anchorage"),
                    ("America/Anguilla", "America/Anguilla"),
                    ("America/Antigua", "America/Antigua"),
                    ("America/Araguaina", "America/Araguaina"),
                    (
                        "America/Argentina/Buenos_Aires",
                        "America/Argentina/Buenos_Aires",
                    ),
                    ("America/Argentina/Catamarca", "America/Argentina/Catamarca"),
                    ("America/Argentina/Cordoba", "America/Argentina/Cordoba"),
                    ("America/Argentina/Jujuy", "America/Argentina/Jujuy"),
                    ("America/Argentina/La_Rioja", "America/Argentina/La_Rioja"),
                    ("America/Argentina/Mendoza", "America/Argentina/Mendoza"),
                    (
                        "America/Argentina/Rio_Gallegos",
                        "America/Argentina/Rio_Gallegos",
                    ),
                    ("America/Argentina/Salta", "America/Argentina/Salta"),
                    ("America/Argentina/San_Juan", "America/Argentina/San_Juan"),
                    ("America/Argentina/San_Luis", "America/Argentina/San_Luis"),
                    ("America/Argentina/Tucuman", "America/Argentina/Tucuman"),
                    ("America/Argentina/Ushuaia", "America/Argentina/Ushuaia"),
                    ("America/Aruba", "America/Aruba"),
                    ("America/Asuncion", "America/Asuncion"),
                    ("America/Atikokan", "America/Atikokan"),
                    ("America/Bahia", "America/Bahia"),
                    ("America/Bahia_Banderas", "America/Bahia_Banderas"),
                    ("America/Barbados", "America/Barbados"),
                    ("America/Belem", "America/Belem"),
                    ("America/Belize", "America/Belize"),
                    ("America/Blanc-Sablon", "America/Blanc-Sablon"),
                    ("America/Boa_Vista", "America/Boa_Vista"),
                    ("America/Bogota", "America/Bogota"),
                    ("America/Boise", "America/Boise"),
                    ("America/Cambridge_Bay", "America/Cambridge_Bay"),
                    ("America/Campo_Grande", "America/Campo_Grande"),
                    ("America/Cancun", "America/Cancun"),
                    ("America/Caracas", "America/Caracas"),
                    ("America/Cayenne", "America/Cayenne"),
                    ("America/Cayman", "America/Cayman"),
                    ("America/Chicago", "America/Chicago"),
                    ("America/Chihuahua", "America/Chihuahua"),
                    ("America/Costa_Rica", "America/Costa_Rica"),
                    ("America/Creston", "America/Creston"),
                    ("America/Cuiaba", "America/Cuiaba"),
                    ("America/Curacao", "America/Curacao"),
                    ("America/Danmarkshavn", "America/Danmarkshavn"),
                    ("America/Dawson", "America/Dawson"),
                    ("America/Dawson_Creek", "America/Dawson_Creek"),
                    ("America/Denver", "America/Denver"),
                    ("America/Detroit", "America/Detroit"),
                    ("America/Dominica", "America/Dominica"),
                    ("America/Edmonton", "America/Edmonton"),
                    ("America/Eirunepe", "America/Eirunepe"),
                    ("America/El_Salvador", "America/El_Salvador"),
                    ("America/Fort_Nelson", "America/Fort_Nelson"),
                    ("America/Fortaleza", "America/Fortaleza"),
                    ("America/Glace_Bay", "America/Glace_Bay"),
                    ("America/Godthab", "America/Godthab"),
                    ("America/Goose_Bay", "America/Goose_Bay"),
                    ("America/Grand_Turk", "America/Grand_Turk"),
                    ("America/Grenada", "America/Grenada"),
                    ("America/Guadeloupe", "America/Guadeloupe"),
                    ("America/Guatemala", "America/Guatemala"),
                    ("America/Guayaquil", "America/Guayaquil"),
                    ("America/Guyana", "America/Guyana"),
                    ("America/Halifax", "America/Halifax"),
                    ("America/Havana", "America/Havana"),
                    ("America/Hermosillo", "America/Hermosillo"),
                    ("America/Indiana/Indianapolis", "America/Indiana/Indianapolis"),
                    ("America/Indiana/Knox", "America/Indiana/Knox"),
                    ("America/Indiana/Marengo", "America/Indiana/Marengo"),
                    ("America/Indiana/Petersburg", "America/Indiana/Petersburg"),
                    ("America/Indiana/Tell_City", "America/Indiana/Tell_City"),
                    ("America/Indiana/Vevay", "America/Indiana/Vevay"),
                    ("America/Indiana/Vincennes", "America/Indiana/Vincennes"),
                    ("America/Indiana/Winamac", "America/Indiana/Winamac"),
                    ("America/Inuvik", "America/Inuvik"),
                    ("America/Iqaluit", "America/Iqaluit"),
                    ("America/Jamaica", "America/Jamaica"),
                    ("America/Juneau", "America/Juneau"),
                    ("America/Kentucky/Louisville", "America/Kentucky/Louisville"),
                    ("America/Kentucky/Monticello", "America/Kentucky/Monticello"),
                    ("America/Kralendijk", "America/Kralendijk"),
                    ("America/La_Paz", "America/La_Paz"),
                    ("America/Lima", "America/Lima"),
                    ("America/Los_Angeles", "America/Los_Angeles"),
                    ("America/Lower_Princes", "America/Lower_Princes"),
                    ("America/Maceio", "America/Maceio"),
                    ("America/Managua", "America/Managua"),
                    ("America/Manaus", "America/Manaus"),
                    ("America/Marigot", "America/Marigot"),
                    ("America/Martinique", "America/Martinique"),
                    ("America/Matamoros", "America/Matamoros"),
                    ("America/Mazatlan", "America/Mazatlan"),
                    ("America/Menominee", "America/Menominee"),
                    ("America/Merida", "America/Merida"),
                    ("America/Metlakatla", "America/Metlakatla"),
                    ("America/Mexico_City", "America/Mexico_City"),
                    ("America/Miquelon", "America/Miquelon"),
                    ("America/Moncton", "America/Moncton"),
                    ("America/Monterrey", "America/Monterrey"),
                    ("America/Montevideo", "America/Montevideo"),
                    ("America/Montserrat", "America/Montserrat"),
                    ("America/Nassau", "America/Nassau"),
                    ("America/New_York", "America/New_York"),
                    ("America/Nipigon", "America/Nipigon"),
                    ("America/Nome", "America/Nome"),
                    ("America/Noronha", "America/Noronha"),
                    ("America/North_Dakota/Beulah", "America/North_Dakota/Beulah"),
                    ("America/North_Dakota/Center", "America/North_Dakota/Center"),
                    (
                        "America/North_Dakota/New_Salem",
                        "America/North_Dakota/New_Salem",
                    ),
                    ("America/Ojinaga", "America/Ojinaga"),
                    ("America/Panama", "America/Panama"),
                    ("America/Pangnirtung", "America/Pangnirtung"),
                    ("America/Paramaribo", "America/Paramaribo"),
                    ("America/Phoenix", "America/Phoenix"),
                    ("America/Port-au-Prince", "America/Port-au-Prince"),
                    ("America/Port_of_Spain", "America/Port_of_Spain"),
                    ("America/Porto_Velho", "America/Porto_Velho"),
                    ("America/Puerto_Rico", "America/Puerto_Rico"),
                    ("America/Punta_Arenas", "America/Punta_Arenas"),
                    ("America/Rainy_River", "America/Rainy_River"),
                    ("America/Rankin_Inlet", "America/Rankin_Inlet"),
                    ("America/Recife", "America/Recife"),
                    ("America/Regina", "America/Regina"),
                    ("America/Resolute", "America/Resolute"),
                    ("America/Rio_Branco", "America/Rio_Branco"),
                    ("America/Santarem", "America/Santarem"),
                    ("America/Santiago", "America/Santiago"),
                    ("America/Santo_Domingo", "America/Santo_Domingo"),
                    ("America/Sao_Paulo", "America/Sao_Paulo"),
                    ("America/Scoresbysund", "America/Scoresbysund"),
                    ("America/Sitka", "America/Sitka"),
                    ("America/St_Barthelemy", "America/St_Barthelemy"),
                    ("America/St_Johns", "America/St_Johns"),
                    ("America/St_Kitts", "America/St_Kitts"),
                    ("America/St_Lucia", "America/St_Lucia"),
                    ("America/St_Thomas", "America/St_Thomas"),
                    ("America/St_Vincent", "America/St_Vincent"),
                    ("America/Swift_Current", "America/Swift_Current"),
                    ("America/Tegucigalpa", "America/Tegucigalpa"),
                    ("America/Thule", "America/Thule"),
                    ("America/Thunder_Bay", "America/Thunder_Bay"),
                    ("America/Tijuana", "America/Tijuana"),
                    ("America/Toronto", "America/Toronto"),
                    ("America/Tortola", "America/Tortola"),
                    ("America/Vancouver", "America/Vancouver"),
                    ("America/Whitehorse", "America/Whitehorse"),
                    ("America/Winnipeg", "America/Winnipeg"),
                    ("America/Yakutat", "America/Yakutat"),
                    ("America/Yellowknife", "America/Yellowknife"),
                    ("Antarctica/Casey", "Antarctica/Casey"),
                    ("Antarctica/Davis", "Antarctica/Davis"),
                    ("Antarctica/DumontDUrville", "Antarctica/DumontDUrville"),
                    ("Antarctica/Macquarie", "Antarctica/Macquarie"),
                    ("Antarctica/Mawson", "Antarctica/Mawson"),
                    ("Antarctica/McMurdo", "Antarctica/McMurdo"),
                    ("Antarctica/Palmer", "Antarctica/Palmer"),
                    ("Antarctica/Rothera", "Antarctica/Rothera"),
                    ("Antarctica/Syowa", "Antarctica/Syowa"),
                    ("Antarctica/Troll", "Antarctica/Troll"),
                    ("Antarctica/Vostok", "Antarctica/Vostok"),
                    ("Arctic/Longyearbyen", "Arctic/Longyearbyen"),
                    ("Asia/Aden", "Asia/Aden"),
                    ("Asia/Almaty", "Asia/Almaty"),
                    ("Asia/Amman", "Asia/Amman"),
                    ("Asia/Anadyr", "Asia/Anadyr"),
                    ("Asia/Aqtau", "Asia/Aqtau"),
                    ("Asia/Aqtobe", "Asia/Aqtobe"),
                    ("Asia/Ashgabat", "Asia/Ashgabat"),
                    ("Asia/Atyrau", "Asia/Atyrau"),
                    ("Asia/Baghdad", "Asia/Baghdad"),
                    ("Asia/Bahrain", "Asia/Bahrain"),
                    ("Asia/Baku", "Asia/Baku"),
                    ("Asia/Bangkok", "Asia/Bangkok"),
                    ("Asia/Barnaul", "Asia/Barnaul"),
                    ("Asia/Beirut", "Asia/Beirut"),
                    ("Asia/Bishkek", "Asia/Bishkek"),
                    ("Asia/Brunei", "Asia/Brunei"),
                    ("Asia/Chita", "Asia/Chita"),
                    ("Asia/Choibalsan", "Asia/Choibalsan"),
                    ("Asia/Colombo", "Asia/Colombo"),
                    ("Asia/Damascus", "Asia/Damascus"),
                    ("Asia/Dhaka", "Asia/Dhaka"),
                    ("Asia/Dili", "Asia/Dili"),
                    ("Asia/Dubai", "Asia/Dubai"),
                    ("Asia/Dushanbe", "Asia/Dushanbe"),
                    ("Asia/Famagusta", "Asia/Famagusta"),
                    ("Asia/Gaza", "Asia/Gaza"),
                    ("Asia/Hebron", "Asia/Hebron"),
                    ("Asia/Ho_Chi_Minh", "Asia/Ho_Chi_Minh"),
                    ("Asia/Hong_Kong", "Asia/Hong_Kong"),
                    ("Asia/Hovd", "Asia/Hovd"),
                    ("Asia/Irkutsk", "Asia/Irkutsk"),
                    ("Asia/Jakarta", "Asia/Jakarta"),
                    ("Asia/Jayapura", "Asia/Jayapura"),
                    ("Asia/Jerusalem", "Asia/Jerusalem"),
                    ("Asia/Kabul", "Asia/Kabul"),
                    ("Asia/Kamchatka", "Asia/Kamchatka"),
                    ("Asia/Karachi", "Asia/Karachi"),
                    ("Asia/Kathmandu", "Asia/Kathmandu"),
                    ("Asia/Khandyga", "Asia/Khandyga"),
                    ("Asia/Kolkata", "Asia/Kolkata"),
                    ("Asia/Krasnoyarsk", "Asia/Krasnoyarsk"),
                    ("Asia/Kuala_Lumpur", "Asia/Kuala_Lumpur"),
                    ("Asia/Kuching", "Asia/Kuching"),
                    ("Asia/Kuwait", "Asia/Kuwait"),
                    ("Asia/Macau", "Asia/Macau"),
                    ("Asia/Magadan", "Asia/Magadan"),
                    ("Asia/Makassar", "Asia/Makassar"),
                    ("Asia/Manila", "Asia/Manila"),
                    ("Asia/Muscat", "Asia/Muscat"),
                    ("Asia/Nicosia", "Asia/Nicosia"),
                    ("Asia/Novokuznetsk", "Asia/Novokuznetsk"),
                    ("Asia/Novosibirsk", "Asia/Novosibirsk"),
                    ("Asia/Omsk", "Asia/Omsk"),
                    ("Asia/Oral", "Asia/Oral"),
                    ("Asia/Phnom_Penh", "Asia/Phnom_Penh"),
                    ("Asia/Pontianak", "Asia/Pontianak"),
                    ("Asia/Pyongyang", "Asia/Pyongyang"),
                    ("Asia/Qatar", "Asia/Qatar"),
                    ("Asia/Qostanay", "Asia/Qostanay"),
                    ("Asia/Qyzylorda", "Asia/Qyzylorda"),
                    ("Asia/Riyadh", "Asia/Riyadh"),
                    ("Asia/Sakhalin", "Asia/Sakhalin"),
                    ("Asia/Samarkand", "Asia/Samarkand"),
                    ("Asia/Seoul", "Asia/Seoul"),
                    ("Asia/Shanghai", "Asia/Shanghai"),
                    ("Asia/Singapore", "Asia/Singapore"),
                    ("Asia/Srednekolymsk", "Asia/Srednekolymsk"),
                    ("Asia/Taipei", "Asia/Taipei"),
                    ("Asia/Tashkent", "Asia/Tashkent"),
                    ("Asia/Tbilisi", "Asia/Tbilisi"),
                    ("Asia/Tehran", "Asia/Tehran"),
                    ("Asia/Thimphu", "Asia/Thimphu"),
                    ("Asia/Tokyo", "Asia/Tokyo"),
                    ("Asia/Tomsk", "Asia/Tomsk"),
                    ("Asia/Ulaanbaatar", "Asia/Ulaanbaatar"),
                    ("Asia/Urumqi", "Asia/Urumqi"),
                    ("Asia/Ust-Nera", "Asia/Ust-Nera"),
                    ("Asia/Vientiane", "Asia/Vientiane"),
                    ("Asia/Vladivostok", "Asia/Vladivostok"),
                    ("Asia/Yakutsk", "Asia/Yakutsk"),
                    ("Asia/Yangon", "Asia/Yangon"),
                    ("Asia/Yekaterinburg", "Asia/Yekaterinburg"),
                    ("Asia/Yerevan", "Asia/Yerevan"),
                    ("Atlantic/Azores", "Atlantic/Azores"),
                    ("Atlantic/Bermuda", "Atlantic/Bermuda"),
                    ("Atlantic/Canary", "Atlantic/Canary"),
                    ("Atlantic/Cape_Verde", "Atlantic/Cape_Verde"),
                    ("Atlantic/Faroe", "Atlantic/Faroe"),
                    ("Atlantic/Madeira", "Atlantic/Madeira"),
                    ("Atlantic/Reykjavik", "Atlantic/Reykjavik"),
                    ("Atlantic/South_Georgia", "Atlantic/South_Georgia"),
                    ("Atlantic/St_Helena", "Atlantic/St_Helena"),
                    ("Atlantic/Stanley", "Atlantic/Stanley"),
                    ("Australia/Adelaide", "Australia/Adelaide"),
                    ("Australia/Brisbane", "Australia/Brisbane"),
                    ("Australia/Broken_Hill", "Australia/Broken_Hill"),
                    ("Australia/Currie", "Australia/Currie"),
                    ("Australia/Darwin", "Australia/Darwin"),
                    ("Australia/Eucla", "Australia/Eucla"),
                    ("Australia/Hobart", "Australia/Hobart"),
                    ("Australia/Lindeman", "Australia/Lindeman"),
                    ("Australia/Lord_Howe", "Australia/Lord_Howe"),
                    ("Australia/Melbourne", "Australia/Melbourne"),
                    ("Australia/Perth", "Australia/Perth"),
                    ("Australia/Sydney", "Australia/Sydney"),
                    ("Canada/Atlantic", "Canada/Atlantic"),
                    ("Canada/Central", "Canada/Central"),
                    ("Canada/Eastern", "Canada/Eastern"),
                    ("Canada/Mountain", "Canada/Mountain"),
                    ("Canada/Newfoundland", "Canada/Newfoundland"),
                    ("Canada/Pacific", "Canada/Pacific"),
                    ("Europe/Amsterdam", "Europe/Amsterdam"),
                    ("Europe/Andorra", "Europe/Andorra"),
                    ("Europe/Astrakhan", "Europe/Astrakhan"),
                    ("Europe/Athens", "Europe/Athens"),
                    ("Europe/Belgrade", "Europe/Belgrade"),
                    ("Europe/Berlin", "Europe/Berlin"),
                    ("Europe/Bratislava", "Europe/Bratislava"),
                    ("Europe/Brussels", "Europe/Brussels"),
                    ("Europe/Bucharest", "Europe/Bucharest"),
                    ("Europe/Budapest", "Europe/Budapest"),
                    ("Europe/Busingen", "Europe/Busingen"),
                    ("Europe/Chisinau", "Europe/Chisinau"),
                    ("Europe/Copenhagen", "Europe/Copenhagen"),
                    ("Europe/Dublin", "Europe/Dublin"),
                    ("Europe/Gibraltar", "Europe/Gibraltar"),
                    ("Europe/Guernsey", "Europe/Guernsey"),
                    ("Europe/Helsinki", "Europe/Helsinki"),
                    ("Europe/Isle_of_Man", "Europe/Isle_of_Man"),
                    ("Europe/Istanbul", "Europe/Istanbul"),
                    ("Europe/Jersey", "Europe/Jersey"),
                    ("Europe/Kaliningrad", "Europe/Kaliningrad"),
                    ("Europe/Kiev", "Europe/Kiev"),
                    ("Europe/Kirov", "Europe/Kirov"),
                    ("Europe/Lisbon", "Europe/Lisbon"),
                    ("Europe/Ljubljana", "Europe/Ljubljana"),
                    ("Europe/London", "Europe/London"),
                    ("Europe/Luxembourg", "Europe/Luxembourg"),
                    ("Europe/Madrid", "Europe/Madrid"),
                    ("Europe/Malta", "Europe/Malta"),
                    ("Europe/Mariehamn", "Europe/Mariehamn"),
                    ("Europe/Minsk", "Europe/Minsk"),
                    ("Europe/Monaco", "Europe/Monaco"),
                    ("Europe/Moscow", "Europe/Moscow"),
                    ("Europe/Oslo", "Europe/Oslo"),
                    ("Europe/Paris", "Europe/Paris"),
                    ("Europe/Podgorica", "Europe/Podgorica"),
                    ("Europe/Prague", "Europe/Prague"),
                    ("Europe/Riga", "Europe/Riga"),
                    ("Europe/Rome", "Europe/Rome"),
                    ("Europe/Samara", "Europe/Samara"),
                    ("Europe/San_Marino", "Europe/San_Marino"),
                    ("Europe/Sarajevo", "Europe/Sarajevo"),
                    ("Europe/Saratov", "Europe/Saratov"),
                    ("Europe/Simferopol", "Europe/Simferopol"),
                    ("Europe/Skopje", "Europe/Skopje"),
                    ("Europe/Sofia", "Europe/Sofia"),
                    ("Europe/Stockholm", "Europe/Stockholm"),
                    ("Europe/Tallinn", "Europe/Tallinn"),
                    ("Europe/Tirane", "Europe/Tirane"),
                    ("Europe/Ulyanovsk", "Europe/Ulyanovsk"),
                    ("Europe/Uzhgorod", "Europe/Uzhgorod"),
                    ("Europe/Vaduz", "Europe/Vaduz"),
                    ("Europe/Vatican", "Europe/Vatican"),
                    ("Europe/Vienna", "Europe/Vienna"),
                    ("Europe/Vilnius", "Europe/Vilnius"),
                    ("Europe/Volgograd", "Europe/Volgograd"),
                    ("Europe/Warsaw", "Europe/Warsaw"),
                    ("Europe/Zagreb", "Europe/Zagreb"),
                    ("Europe/Zaporozhye", "Europe/Zaporozhye"),
                    ("Europe/Zurich", "Europe/Zurich"),
                    ("GMT", "GMT"),
                    ("Indian/Antananarivo", "Indian/Antananarivo"),
                    ("Indian/Chagos", "Indian/Chagos"),
                    ("Indian/Christmas", "Indian/Christmas"),
                    ("Indian/Cocos", "Indian/Cocos"),
                    ("Indian/Comoro", "Indian/Comoro"),
                    ("Indian/Kerguelen", "Indian/Kerguelen"),
                    ("Indian/Mahe", "Indian/Mahe"),
                    ("Indian/Maldives", "Indian/Maldives"),
                    ("Indian/Mauritius", "Indian/Mauritius"),
                    ("Indian/Mayotte", "Indian/Mayotte"),
                    ("Indian/Reunion", "Indian/Reunion"),
                    ("Pacific/Apia", "Pacific/Apia"),
                    ("Pacific/Auckland", "Pacific/Auckland"),
                    ("Pacific/Bougainville", "Pacific/Bougainville"),
                    ("Pacific/Chatham", "Pacific/Chatham"),
                    ("Pacific/Chuuk", "Pacific/Chuuk"),
                    ("Pacific/Easter", "Pacific/Easter"),
                    ("Pacific/Efate", "Pacific/Efate"),
                    ("Pacific/Enderbury", "Pacific/Enderbury"),
                    ("Pacific/Fakaofo", "Pacific/Fakaofo"),
                    ("Pacific/Fiji", "Pacific/Fiji"),
                    ("Pacific/Funafuti", "Pacific/Funafuti"),
                    ("Pacific/Galapagos", "Pacific/Galapagos"),
                    ("Pacific/Gambier", "Pacific/Gambier"),
                    ("Pacific/Guadalcanal", "Pacific/Guadalcanal"),
                    ("Pacific/Guam", "Pacific/Guam"),
                    ("Pacific/Honolulu", "Pacific/Honolulu"),
                    ("Pacific/Kiritimati", "Pacific/Kiritimati"),
                    ("Pacific/Kosrae", "Pacific/Kosrae"),
                    ("Pacific/Kwajalein", "Pacific/Kwajalein"),
                    ("Pacific/Majuro", "Pacific/Majuro"),
                    ("Pacific/Marquesas", "Pacific/Marquesas"),
                    ("Pacific/Midway", "Pacific/Midway"),
                    ("Pacific/Nauru", "Pacific/Nauru"),
                    ("Pacific/Niue", "Pacific/Niue"),
                    ("Pacific/Norfolk", "Pacific/Norfolk"),
                    ("Pacific/Noumea", "Pacific/Noumea"),
                    ("Pacific/Pago_Pago", "Pacific/Pago_Pago"),
                    ("Pacific/Palau", "Pacific/Palau"),
                    ("Pacific/Pitcairn", "Pacific/Pitcairn"),
                    ("Pacific/Pohnpei", "Pacific/Pohnpei"),
                    ("Pacific/Port_Moresby", "Pacific/Port_Moresby"),
                    ("Pacific/Rarotonga", "Pacific/Rarotonga"),
                    ("Pacific/Saipan", "Pacific/Saipan"),
                    ("Pacific/Tahiti", "Pacific/Tahiti"),
                    ("Pacific/Tarawa", "Pacific/Tarawa"),
                    ("Pacific/Tongatapu", "Pacific/Tongatapu"),
                    ("Pacific/Wake", "Pacific/Wake"),
                    ("Pacific/Wallis", "Pacific/Wallis"),
                    ("US/Alaska", "US/Alaska"),
                    ("US/Arizona", "US/Arizona"),
                    ("US/Central", "US/Central"),
                    ("US/Eastern", "US/Eastern"),
                    ("US/Hawaii", "US/Hawaii"),
                    ("US/Mountain", "US/Mountain"),
                    ("US/Pacific", "US/Pacific"),
                    ("UTC", "UTC"),
                ],
                default="UTC",
                help_text="The time zone of the DMS. Only needed when using Browser binding.",
                max_length=200,
            ),
        ),
        migrations.CreateModel(
            name="UrlMapping",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "long_pattern",
                    models.CharField(
                        help_text="Part of the URL to replace.",
                        max_length=1000,
                        unique=True,
                        verbose_name="Long pattern",
                    ),
                ),
                (
                    "short_pattern",
                    models.CharField(
                        help_text="Replacement string to make the URL shorter.",
                        max_length=18,
                        unique=True,
                        verbose_name="Short pattern",
                    ),
                ),
                (
                    "config",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="drc_cmis.CMISConfig",
                    ),
                ),
            ],
            options={
                "verbose_name": "URL mapping",
                "verbose_name_plural": "URL mappings",
            },
        ),
    ]



from django.contrib import admin
from django.urls import include, path
from django.views.generic import TemplateView

urlpatterns = [
    path("admin/", admin.site.urls),
    path(
        "test-url/<version>/<uuid>",
        TemplateView.as_view(),
        name="enkelvoudiginformatieobject-detail",
    ),
    path(
        "test-url/<version>/<uuid>/download/",
        TemplateView.as_view(),
        name="enkelvoudiginformatieobject-download",
    ),
    path(
        "test-url2/<version>/<uuid>",
        TemplateView.as_view(),
        name="objectinformatieobject-detail",
    ),
    path("ref/", include("vng_api_common.notifications.urls")),
]



# Generated by Django 2.2.10 on 2020-06-16 09:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("drc_cmis", "0003_auto_20191029_1349"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="cmisconfig",
            name="locations",
        ),
        migrations.AddField(
            model_name="cmisconfig",
            name="base_folder",
            field=models.CharField(
                default="DRC",
                help_text="Name of the DMS folder in which the documents will be stored.",
                max_length=200,
            ),
        ),
        migrations.AlterField(
            model_name="cmisconfig",
            name="client_password",
            field=models.CharField(
                default="admin",
                help_text="Password for logging into DMS",
                max_length=200,
            ),
        ),
        migrations.AlterField(
            model_name="cmisconfig",
            name="client_url",
            field=models.URLField(
                default="http://localhost:8082/alfresco/api/-default-/public/cmis/versions/1.1/browser",
                help_text=(
                    "API URL for DMS. For example, for alfresco this can be http://domain_name:port_number/"
                    "alfresco/api/-default-/public/cmis/versions/1.1/browser"
                ),
            ),
        ),
        migrations.AlterField(
            model_name="cmisconfig",
            name="client_user",
            field=models.CharField(
                default="admin",
                help_text="Username for logging into DMS",
                max_length=200,
            ),
        ),
        migrations.DeleteModel(
            name="CMISFolderLocation",
        ),
    ]



# Generated by Django 2.2.10 on 2020-07-29 05:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("drc_cmis", "0005_auto_20200723_1352"),
    ]

    operations = [
        migrations.AddField(
            model_name="cmisconfig",
            name="binding",
            field=models.CharField(
                choices=[
                    ("BROWSER", "Browser binding (CMIS 1.1)"),
                    ("WEBSERVICE", "Web service binding (CMIS 1.0)"),
                ],
                default="BROWSER",
                max_length=200,
            ),
        ),
    ]



import re

from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.utils.translation import ugettext_lazy as _

import pytz
from djchoices import ChoiceItem, DjangoChoices
from solo.models import SingletonModel

from .utils.exceptions import NoOtherBaseFolderException, NoZaakBaseFolderException
from .utils.folder import get_folder_structure
from .validators import (
    other_folder_path_validator,
    url_mapping_validator,
    zaak_folder_path_validator,
)


class CMISConfig(SingletonModel):
    BINDING_CHOICES = [
        ("BROWSER", "Browser binding (CMIS 1.1)"),
        ("WEBSERVICE", "Web service binding (CMIS 1.0)"),
    ]

    client_url = models.URLField(
        default="http://localhost:8082/alfresco/api/-default-/public/cmis/versions/1.1/browser",
        help_text="API URL for DMS. For example, for alfresco this can be "
        "http://domain_name:port_number/alfresco/api/-default-/public/cmis/versions/1.1/browser",
    )
    binding = models.CharField(
        choices=BINDING_CHOICES,
        default="BROWSER",
        max_length=200,
    )
    time_zone = models.CharField(
        default="UTC",
        choices=[(k, k) for k in pytz.common_timezones],
        max_length=200,
        help_text="The time zone of the DMS. Only needed when using Browser binding.",
    )
    main_repo_id = models.CharField(
        max_length=200,
        blank=True,
        help_text=_(
            "ID of the main repository in the CMS. "
            "Only needed if there are multiple repositories present and Webservice binding is used."
        ),
        verbose_name=_("Main repository ID"),
    )
    client_user = models.CharField(
        max_length=200, default="admin", help_text="Username for logging into DMS"
    )
    client_password = models.CharField(
        max_length=200, default="admin", help_text="Password for logging into DMS"
    )
    zaak_folder_path = models.CharField(
        max_length=500,
        default="/DRC/{{ zaaktype }}/{{ year }}/{{ month }}/{{ day }}/{{ zaak }}/",
        validators=[zaak_folder_path_validator],
        help_text=_("The path where documents related to zaken are saved."),
    )
    other_folder_path = models.CharField(
        max_length=500,
        default="/DRC/{{ year }}/{{ month }}/{{ day }}/",
        validators=[other_folder_path_validator],
        help_text=_("The path where other documents are saved."),
    )

    def __str__(self):
        return "CMIS Configuration"

    class Meta:
        verbose_name = "CMIS Configuration"

    def get_zaak_base_folder_name(self) -> str:
        folders = get_folder_structure(self.zaak_folder_path)
        if len(folders) > 0:
            folder_name = re.search(r"^{{ (.+?) }}$", folders[0].folder_name)
            if folder_name is not None:
                raise NoZaakBaseFolderException("No zaak base folder in use.")
            return folders[0].folder_name
        raise ImproperlyConfigured("The 'zaak_folder_path' must be configured.")

    def get_other_base_folder_name(self) -> str:
        folders = get_folder_structure(self.other_folder_path)
        if len(folders) > 0:
            folder_name = re.search(r"^{{ (.+?) }}$", folders[0].folder_name)
            if folder_name is not None:
                raise NoOtherBaseFolderException("No other base folder in use.")
            return folders[0].folder_name
        raise ImproperlyConfigured("The 'other_folder_path' must be configured.")


class Vendor(DjangoChoices):
    alfresco = ChoiceItem()
    bct = ChoiceItem()


class UrlMapping(models.Model):
    # Cant use a URL field, because the pattern could be http://testserver, which is an invalid URL
    long_pattern = models.CharField(
        max_length=1000,
        verbose_name=_("Long pattern"),
        help_text=_(
            "Part of the URL to replace. Example: https://somedomain.nl/somesubpath"
        ),
        unique=True,
        validators=[url_mapping_validator],
    )
    short_pattern = models.CharField(
        max_length=15,
        verbose_name=_("Short pattern"),
        help_text=_(
            "Replacement string to make the URL shorter. Example: https://sd.nl"
        ),
        unique=True,
        validators=[url_mapping_validator],
    )
    config = models.ForeignKey(
        to=CMISConfig,
        on_delete=models.CASCADE,
    )

    class Meta:
        verbose_name = _("URL mapping")
        verbose_name_plural = _("URL mappings")

    def __string__(self):
        return f"{self.long_pattern}: {self.short_pattern}"



import logging
from datetime import datetime
from typing import Optional

from django.conf import settings
from django.contrib.sites.models import Site
from django.http import HttpRequest
from django.urls import reverse

import iso8601

logger = logging.getLogger(__name__)


def make_absolute_uri(path: str, request: Optional[HttpRequest] = None) -> str:
    if request is not None:
        return request.build_absolute_uri(path)

    site = Site.objects.get_current()
    protocol = "https" if getattr(settings, "IS_HTTPS", True) else "http"
    return f"{protocol}://{site.domain}{path}"


def to_date(value):
    tmp_datetime = parseDateTimeValue(value)
    if tmp_datetime:
        return tmp_datetime.date()
    return None


def to_datetime(value):
    tmp_datetime = parseDateTimeValue(value)
    return tmp_datetime


def parseDateTimeValue(value):
    """
    Utility function to return a datetime from a string.
    """
    if type(value) == str:
        return iso8601.parse_date(value)
    elif type(value) == int:
        return datetime.fromtimestamp(value / 1000)
    else:
        return None


def make_enkelvoudiginformatieobject_dataclass(cmis_doc, dataclass, skip_deleted=False):
    if cmis_doc.verwijderd and not skip_deleted:
        # Return None if document is deleted.
        return None

    path = reverse(
        "enkelvoudiginformatieobject-detail",
        kwargs={"version": "1", "uuid": cmis_doc.versionSeriesId},
    )
    url = make_absolute_uri(path)

    download_link = reverse(
        "enkelvoudiginformatieobject-download",
        kwargs={"version": "1", "uuid": cmis_doc.versionSeriesId},
    )
    download_url = make_absolute_uri(download_link)

    return dataclass(
        url=url,
        inhoud=download_url,
        creatiedatum=to_date(cmis_doc.creatiedatum),
        ontvangstdatum=to_date(cmis_doc.ontvangstdatum),
        verzenddatum=to_date(cmis_doc.verzenddatum),
        integriteit_datum=to_date(cmis_doc.integriteit_datum),
        ondertekening_datum=to_date(cmis_doc.ondertekening_datum),
        titel=cmis_doc.titel,
        identificatie=cmis_doc.identificatie,
        bronorganisatie=cmis_doc.bronorganisatie,
        vertrouwelijkheidaanduiding=cmis_doc.vertrouwelijkheidaanduiding,
        auteur=cmis_doc.auteur,
        # FIXME -> status must be saved in EIO
        status=cmis_doc.status or "gearchiveerd",
        beschrijving=cmis_doc.beschrijving or "",
        indicatie_gebruiksrecht=cmis_doc.indicatie_gebruiksrecht,
        ondertekening_soort=cmis_doc.ondertekening_soort,
        informatieobjecttype=cmis_doc.informatieobjecttype,
        formaat=cmis_doc.formaat or "",
        taal=cmis_doc.taal,
        bestandsnaam=cmis_doc.bestandsnaam or "",
        link=cmis_doc.link or "",
        integriteit_algoritme=cmis_doc.integriteit_algoritme,
        integriteit_waarde=cmis_doc.integriteit_waarde,
        bestandsomvang=cmis_doc.bestandsomvang,
        begin_registratie=to_datetime(
            cmis_doc.begin_registratie or cmis_doc.creationDate
        ),
        versie=cmis_doc.versie,
        locked=bool(cmis_doc.versionSeriesCheckedOutId),
    )


def make_objectinformatieobject_dataclass(cmis_doc, dataclass, skip_deleted=False):
    if cmis_doc.verwijderd and not skip_deleted:
        # Return None if document is deleted.
        return None

    path = reverse(
        "objectinformatieobject-detail",
        kwargs={"version": "1", "uuid": cmis_doc.versionSeriesId},
    )
    url = make_absolute_uri(path)

    eio_path = reverse(
        "enkelvoudiginformatieobject-detail",
        kwargs={"version": "1", "uuid": cmis_doc.versionSeriesId},
    )
    eio_url = make_absolute_uri(eio_path)

    return dataclass(
        url=url,
        informatieobject=eio_url,
        object=cmis_doc.object,
        object_type=cmis_doc.object_type,
        aard_relatie=cmis_doc.aard_relatie,
        titel=cmis_doc.connectie__titel,
        beschrijving=cmis_doc.connectie__beschrijving,
        registratiedatum=cmis_doc.registratiedatum,
    )



import io
import uuid

from django.core.management.base import BaseCommand
from django.utils import timezone

from drc_cmis.client_builder import get_cmis_client
from drc_cmis.models import CMISConfig
from drc_cmis.utils.utils import get_random_string


class Command(BaseCommand):
    help = "Try all the CMIS operations used in drc_cmis against a DMS"

    def add_arguments(self, parser):
        parser.add_argument("client_url", type=str)
        parser.add_argument("binding", type=str)
        parser.add_argument("client_user", type=str)
        parser.add_argument("client_password", type=str)
        parser.add_argument("zaak_folder_path", type=str)
        parser.add_argument("other_folder_path", type=str)

    def handle(self, *args, **options):
        if CMISConfig.objects.count() == 0:
            CMISConfig.objects.create(
                client_url=options["client_url"],
                binding=options["binding"],
                client_user=options["client_user"],
                client_password=options["client_password"],
                zaak_folder_path=options["zaak_folder_path"],
                other_folder_path=options["other_folder_path"],
            )
        else:
            config = CMISConfig.objects.get()
            config.client_url = options["client_url"]
            config.binding = options["binding"]
            config.client_user = options["client_user"]
            config.client_password = options["client_password"]
            config.other_folder_path = options["other_folder_path"]
            config.zaak_folder_path = options["zaak_folder_path"]
            config.save()

        # General
        cmis_client = get_cmis_client()
        cmis_client.get_repository_info()
        get_root_folder_id(cmis_client)
        get_other_base_folder(cmis_client)
        print("General: Success")

        try:
            # Folders
            get_folder(cmis_client)
            create_folder(cmis_client)
            delete_folder(cmis_client)
            get_or_create_folder(cmis_client)
            create_zaaktype_folder(cmis_client)
            create_zaak_folder(cmis_client)
            create_cmis_folder_in_zaaktype(cmis_client)
            create_cmis_folder_in_zaak(cmis_client)
            print("Folders: Success")

            # Content objects
            create_content_object_oio(cmis_client)
            create_content_object_gebruiksrechten(cmis_client)
            get_content_object_oio(cmis_client)
            get_content_object_gebruiksrechten(cmis_client)
            delete_content_object_oio(cmis_client)
            delete_content_object_gebruiksrechten(cmis_client)
            print("Content objects: Success")

            # Documents
            create_document(cmis_client)
            lock_unlock_document(cmis_client)
            get_pwc(cmis_client)
            update_document(cmis_client)
            create_document_copy(cmis_client)
            move_document(cmis_client)
            delete_document(cmis_client)
            print("Documents: Success")
        except Exception as exc:
            # Clean up
            cmis_client.get_or_create_other_folder().delete_tree()
            print("Cleaned up")
            raise exc

        # Clean up
        cmis_client.get_or_create_other_folder().delete_tree()
        print("Cleaned up")


# ---* General *---
def get_other_base_folder(client):
    client.get_or_create_other_folder()


def get_root_folder_id(client):
    client.root_folder_id


# ---* Folders *---
def get_folder(client):
    client.get_folder(client.root_folder_id)


def create_folder(client):
    client.create_folder(
        f"TestFolder-{get_random_string()}",
        client.get_or_create_other_folder().objectId,
    )


def get_or_create_folder(client):
    client.get_or_create_folder(
        f"TestFolder-{get_random_string()}", client.get_or_create_other_folder()
    )


def create_zaaktype_folder(client):
    config = CMISConfig.objects.get()
    if config.binding == "BROWSER":
        properties = {
            "drc:zaaktype__url": "https://ref.tst.vng.cloud/ztc/api/v1/zaaktypen/0119dd4e-7be9-477e-bccf-75023b1453c1",
            "drc:zaaktype__identificatie": "1",
            "cmis:objectTypeId": f"{client.get_object_type_id_prefix('zaaktypefolder')}drc:zaaktypefolder",
        }
    elif config.binding == "WEBSERVICE":
        properties = {
            "drc:zaaktype__url": {
                "value": "https://ref.tst.vng.cloud/ztc/api/v1/zaaktypen/0119dd4e-7be9-477e-bccf-75023b1453c1",
                "type": "propertyString",
            },
            "drc:zaaktype__identificatie": {"value": "1", "type": "propertyString"},
            "cmis:objectTypeId": {
                "value": f"{client.get_object_type_id_prefix('zaaktypefolder')}drc:zaaktypefolder",
                "type": "propertyId",
            },
        }

    return client.create_folder(
        f"TestZaaktypeFolder-{get_random_string()}",
        client.get_or_create_other_folder().objectId,
        properties,
    )


def create_zaak_folder(client):
    config = CMISConfig.objects.get()
    if config.binding == "BROWSER":
        properties = {
            "drc:zaaktype__url": "https://ref.tst.vng.cloud/ztc/api/v1/zaaktypen/0119dd4e-7be9-477e-bccf-75023b1453c1",
            "drc:zaaktype__identificatie": "1",
            "cmis:objectTypeId": f"{client.get_object_type_id_prefix('zaaktypefolder')}drc:zaaktypefolder",
        }
    elif config.binding == "WEBSERVICE":
        properties = {
            "drc:zaaktype__url": {
                "value": "https://ref.tst.vng.cloud/ztc/api/v1/zaaktypen/0119dd4e-7be9-477e-bccf-75023b1453c1",
                "type": "propertyString",
            },
            "drc:zaaktype__identificatie": {"value": "1", "type": "propertyString"},
            "cmis:objectTypeId": {
                "value": f"{client.get_object_type_id_prefix('zaaktypefolder')}drc:zaaktypefolder",
                "type": "propertyId",
            },
        }
    return client.create_folder(
        f"TestZaakFolder-{get_random_string()}",
        client.get_or_create_other_folder().objectId,
        properties,
    )


def create_cmis_folder_in_zaaktype(client):
    zaaktype_folder = create_zaaktype_folder(client)
    client.create_folder(f"TestFolder-{get_random_string()}", zaaktype_folder.objectId)


def create_cmis_folder_in_zaak(client):
    zaak_folder = create_zaak_folder(client)
    client.create_folder(f"TestFolder-{get_random_string()}", zaak_folder.objectId)


def delete_folder(client):
    folder = client.create_folder(
        f"TestFolder-{get_random_string()}",
        client.get_or_create_other_folder().objectId,
    )
    folder.delete_tree()


def get_or_create_zaaktype_folder(client):
    zaak_type = {
        "url": "https://ref.tst.vng.cloud/ztc/api/v1/zaaktypen/0119dd4e-7be9-477e-bccf-75023b1453c1",
        "identificatie": 1,
        "omschrijving": "Melding Openbare Ruimte",
    }

    client.get_or_create_zaaktype_folder(zaak_type)


# ---* Content Objects *---
def create_content_object_oio(client):
    oio_data = {
        "object": "https://testserver/api/v1/zaken/12345",
        "informatieobject": "https://testserver/api/v1/documenten/12345",
        "object_type": "zaak",
    }
    client.create_content_object(data=oio_data, object_type="oio")


def create_content_object_gebruiksrechten(client):
    gebruiksrechten_data = {
        "informatieobject": "https://testserver/api/v1/documenten/12345",
        "startdatum": "2018-12-24T00:00:00Z",
        "omschrijving_voorwaarden": "Een hele set onredelijke voorwaarden",
    }

    client.create_content_object(
        data=gebruiksrechten_data, object_type="gebruiksrechten"
    )


def get_content_object_oio(client):
    oio_data = {
        "object": "https://testserver/api/v1/zaken/12345",
        "informatieobject": "https://testserver/api/v1/documenten/12345",
        "object_type": "zaak",
    }
    oio = client.create_content_object(data=oio_data, object_type="oio")

    client.get_content_object(drc_uuid=oio.uuid, object_type="oio")


def get_content_object_gebruiksrechten(client):
    gebruiksrechten_data = {
        "informatieobject": "https://testserver/api/v1/documenten/12345",
        "startdatum": "2018-12-24T00:00:00Z",
        "omschrijving_voorwaarden": "Een hele set onredelijke voorwaarden",
    }

    gebruiksrechten = client.create_content_object(
        data=gebruiksrechten_data, object_type="gebruiksrechten"
    )

    client.get_content_object(
        drc_uuid=gebruiksrechten.uuid, object_type="gebruiksrechten"
    )


def delete_content_object_oio(client):
    oio_data = {
        "object": "https://testserver/api/v1/zaken/12345",
        "informatieobject": "https://testserver/api/v1/documenten/12345",
        "object_type": "zaak",
    }
    oio = client.create_content_object(data=oio_data, object_type="oio")

    oio.delete_object()


def delete_content_object_gebruiksrechten(client):
    gebruiksrechten_data = {
        "informatieobject": "https://testserver/api/v1/documenten/12345",
        "startdatum": "2018-12-24T00:00:00Z",
        "omschrijving_voorwaarden": "Een hele set onredelijke voorwaarden",
    }

    gebruiksrechten = client.create_content_object(
        data=gebruiksrechten_data, object_type="gebruiksrechten"
    )

    gebruiksrechten.delete_object()


# ---* Documents *---
def create_document(client):
    data = {
        "creatiedatum": timezone.now(),
        "titel": "detailed summary",
    }
    content = io.BytesIO(b"Test content")
    client.create_document(identification=uuid.uuid4(), data=data, content=content)


def lock_unlock_document(client):
    data = {
        "creatiedatum": timezone.now(),
        "titel": "detailed summary",
    }
    content = io.BytesIO(b"Test content")
    document = client.create_document(
        identification=uuid.uuid4(), data=data, content=content
    )
    client.lock_document(
        drc_uuid=document.uuid, lock="00569792-f72f-420c-8b72-9c2fb9dd7601"
    )
    client.unlock_document(
        drc_uuid=document.uuid, lock="00569792-f72f-420c-8b72-9c2fb9dd7601"
    )


def get_pwc(client):
    data = {
        "creatiedatum": timezone.now(),
        "titel": "detailed summary",
    }
    content = io.BytesIO(b"Test content")
    document = client.create_document(
        identification=uuid.uuid4(), data=data, content=content
    )
    client.lock_document(
        drc_uuid=document.uuid, lock="00569792-f72f-420c-8b72-9c2fb9dd7601"
    )
    document.get_private_working_copy()


def update_document(client):
    data = {
        "creatiedatum": timezone.now(),
        "titel": "detailed summary",
    }
    content = io.BytesIO(b"Content before update")
    document = client.create_document(
        identification=uuid.uuid4(), data=data, content=content
    )

    new_data = {
        "auteur": "updated auteur",
        "link": "http://an.updated.link",
        "beschrijving": "updated beschrijving",
    }
    new_content = io.BytesIO(b"Content after update")

    client.lock_document(
        drc_uuid=document.uuid, lock="00569792-f72f-420c-8b72-9c2fb9dd7601"
    )
    client.update_document(
        drc_uuid=document.uuid,
        data=new_data,
        content=new_content,
        lock="00569792-f72f-420c-8b72-9c2fb9dd7601",
    )
    client.unlock_document(
        drc_uuid=document.uuid, lock="00569792-f72f-420c-8b72-9c2fb9dd7601"
    )


def create_document_copy(client):
    data = {
        "creatiedatum": timezone.now(),
        "titel": "detailed summary",
    }
    content = io.BytesIO(b"Test content")
    document = client.create_document(
        identification=uuid.uuid4(), data=data, content=content
    )

    client.copy_document(document, client.get_or_create_other_folder())


def move_document(client):
    data = {
        "creatiedatum": timezone.now(),
        "titel": "detailed summary",
    }
    content = io.BytesIO(b"Test content")
    document = client.create_document(
        identification=uuid.uuid4(), data=data, content=content
    )
    folder = client.create_folder(
        "TestFolderForMove", client.get_or_create_other_folder().objectId
    )
    document.move_object(folder)


def delete_document(client):
    data = {
        "creatiedatum": timezone.now(),
        "titel": "detailed summary",
    }
    content = io.BytesIO(b"Test content")
    document = client.create_document(
        identification=uuid.uuid4(), data=data, content=content
    )
    client.delete_document(drc_uuid=document.uuid)



from typing import Type, Union

from django.conf import settings
from django.utils.module_loading import import_string

from vng_api_common.models import APICredential
from zds_client.client import Client

from drc_cmis.browser.client import CMISDRCClient
from drc_cmis.models import CMISConfig
from drc_cmis.webservice.client import SOAPCMISClient

try:
    from zgw_consumers.client import get_client_class
except ImportError:

    def get_client_class() -> Type[Client]:
        return Client


def get_cmis_client() -> Union[CMISDRCClient, SOAPCMISClient]:
    """Build the CMIS client with the binding specified in the configuration"""
    config = CMISConfig.get_solo()
    if config.binding == "WEBSERVICE":
        return SOAPCMISClient()
    else:
        return CMISDRCClient()


def get_zds_client(url: str):
    """
    Retrieve a ZDS Client instance for the given API URL.

    The client respects the setting ``CUSTOM_CLIENT_FETCHER`` as used by vng-api-common,
    and will have the auth configured.
    """
    Client = get_client_class()
    default = Client.from_url(url)

    if getattr(settings, "CUSTOM_CLIENT_FETCHER", None):
        client = import_string(settings.CUSTOM_CLIENT_FETCHER)(default.base_url)
        return client
    else:
        default.auth = APICredential.get_auth(url)
        return default



# Generated by Django 2.2.15 on 2020-08-21 15:12

from django.db import migrations, models

import drc_cmis.validators


class Migration(migrations.Migration):

    dependencies = [
        ("drc_cmis", "0011_auto_20200813_1640"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="cmisconfig",
            name="base_folder_name",
        ),
        migrations.AddField(
            model_name="cmisconfig",
            name="other_folder_path",
            field=models.CharField(
                default="/DRC/{{ year }}/{{ month }}/{{ day }}/",
                help_text="The path where other documents are saved.",
                max_length=500,
                validators=[drc_cmis.validators.other_folder_path_validator],
            ),
        ),
        migrations.AddField(
            model_name="cmisconfig",
            name="zaak_folder_path",
            field=models.CharField(
                default="/DRC/{{ zaaktype }}/{{ year }}/{{ month }}/{{ day }}/{{ zaak }}/",
                help_text="The path where documents related to zaken are saved.",
                max_length=500,
                validators=[drc_cmis.validators.zaak_folder_path_validator],
            ),
        ),
    ]



"""
Listen to the notifications that are send by the NRC
"""
from vng_api_common.notifications.handlers import RoutingHandler, default

from drc_cmis.client_builder import get_cmis_client, get_zds_client


class ZakenHandler:
    def __init__(self):
        self.cmis_client = get_cmis_client()

    def handle(self, data: dict) -> None:
        if data.get("actie") == "create" and data.get("resource") == "zaak":
            zaaktype_url = data.get("kenmerken", {}).get("zaaktype")
            zaak_url = data.get("resource_url")

            if zaak_url and zaaktype_url:
                self.create_zaak_folder(zaak_url, zaaktype_url)

    def create_zaak_folder(self, zaak_url: str, zaaktype_url: str, zaaktype_folder):
        client = get_zds_client(zaaktype_url)
        zaaktype_data = client.retrieve("zaaktype", url=zaaktype_url)

        client = get_zds_client(zaak_url)
        zaak_data = client.retrieve("zaak", url=zaak_url)
        self.cmis_client.get_or_create_zaak_folder(zaaktype_data, zaak_data)


default = RoutingHandler({"zaken": ZakenHandler()}, default=default)



# Generated by Django 2.2.17 on 2021-02-04 10:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("drc_cmis", "0013_auto_20201216_1046"),
        ("drc_cmis", "0014_auto_20210106_1602"),
    ]

    operations = []



from urllib.parse import urlparse

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from drc_cmis.utils import folder
from drc_cmis.utils.folder import get_folder_structure


def folder_path_validator(
    path: str, path_element_templates: list = None, required: bool = True
):
    allowed_folder_templates = {pet.folder_name for pet in path_element_templates}
    required_folder_names = {
        pet.folder_name for pet in path_element_templates if pet.required
    }
    count = 0

    for pe in get_folder_structure(path):
        count += 1

        if "{{" in pe.folder_name and pe.folder_name not in allowed_folder_templates:
            raise ValidationError(
                "Invalid templated path element: %(value)s",
                params={"value": pe.folder_name},
            )
        if pe.folder_name in required_folder_names:
            required_folder_names -= {pe.folder_name}

    if len(required_folder_names) != 0:
        raise ValidationError(
            "Required path elements are missing: %(value)s",
            params={"value": ", ".join(required_folder_names)},
        )

    if required and count == 0:
        raise ValidationError("At minimum, one folder is required.")


def zaak_folder_path_validator(path: str):
    path_element_templates = [
        folder.YEAR_PATH_ELEMENT_TEMPLATE,
        folder.MONTH_PATH_ELEMENT_TEMPLATE,
        folder.DAY_PATH_ELEMENT_TEMPLATE,
        folder.ZAAKTYPE_PATH_ELEMENT_TEMPLATE,
        folder.ZAAK_PATH_ELEMENT_TEMPLATE,
    ]
    folder_path_validator(path, path_element_templates)


def other_folder_path_validator(path: str):
    path_element_templates = [
        folder.YEAR_PATH_ELEMENT_TEMPLATE,
        folder.MONTH_PATH_ELEMENT_TEMPLATE,
        folder.DAY_PATH_ELEMENT_TEMPLATE,
    ]
    folder_path_validator(path, path_element_templates)


def url_mapping_validator(pattern: str):
    parsed_url = urlparse(pattern)

    if parsed_url.scheme == "":
        raise ValidationError(
            _("The pattern should start with the protocol (e.g. http:// or https://).")
        )

    if parsed_url.netloc == "":
        raise ValidationError(_("The pattern include a domain name."))



# Generated by Django 2.2.10 on 2020-08-03 07:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("drc_cmis", "0006_cmisconfig_binding"),
    ]

    operations = [
        migrations.AddField(
            model_name="cmisconfig",
            name="time_zone",
            field=models.CharField(
                default="UTC",
                help_text="The time zone of the DMS. Only needed when using Browser binding.",
                max_length=200,
            ),
        ),
    ]



import logging
from json.decoder import JSONDecodeError

import requests

from drc_cmis.utils.exceptions import (
    CmisBaseException,
    CmisInvalidArgumentException,
    CmisNotSupportedException,
    CmisNoValidResponse,
    CmisObjectNotFoundException,
    CmisPermissionDeniedException,
    CmisRuntimeException,
    CmisUpdateConflictException,
    GetFirstException,
)

logger = logging.getLogger(__name__)


class CMISRequest:
    @property
    def config(self):
        """
        Lazily load the config so that no DB queries are done while Django is starting.
        """
        from drc_cmis.models import CMISConfig

        return CMISConfig.get_solo()

    @property
    def base_url(self):
        return self.config.client_url

    @property
    def time_zone(self):
        return self.config.time_zone

    @property
    def root_folder_url(self):
        return f"{self.base_url}/root"

    @property
    def user(self):
        return self.config.client_user

    @property
    def password(self):
        return self.config.client_password

    def get_request(self, url, params=None):
        logger.debug(f"GET: {url} | {params}")
        headers = {"Accept": "application/json"}
        response = requests.get(
            url, params=params, auth=(self.user, self.password), headers=headers
        )
        if not response.ok:
            raise Exception("Error with the query")

        if response.headers.get("Content-Type").startswith("application/json"):
            return response.json()
        return response.content

    def post_request(self, url, data, headers=None, files=None):
        logger.debug(f"POST: {url} | {data}")
        if headers is None:
            headers = {"Accept": "application/json"}
        response = requests.post(
            url,
            data=data,
            auth=(self.user, self.password),
            files=files,
            headers=headers,
        )
        if not response.ok:
            error = response.json()
            if response.status_code == 401:
                raise CmisPermissionDeniedException(
                    status=response.status_code,
                    url=url,
                    message=error.get("message"),
                    code=error.get("exception"),
                )
            elif response.status_code == 400:
                raise CmisInvalidArgumentException(
                    status=response.status_code,
                    url=url,
                    message=error.get("message"),
                    code=error.get("exception"),
                )
            elif response.status_code == 404:
                raise CmisObjectNotFoundException(
                    status=response.status_code,
                    url=url,
                    message=error.get("message"),
                    code=error.get("exception"),
                )
            elif response.status_code == 403:
                raise CmisPermissionDeniedException(
                    status=response.status_code,
                    url=url,
                    message=error.get("message"),
                    code=error.get("exception"),
                )
            elif response.status_code == 405:
                raise CmisNotSupportedException(
                    status=response.status_code,
                    url=url,
                    message=error.get("message"),
                    code=error.get("exception"),
                )
            elif response.status_code == 409:
                raise CmisUpdateConflictException(
                    status=response.status_code,
                    url=url,
                    message=error.get("message"),
                    code=error.get("exception"),
                )
            elif response.status_code == 500:
                raise CmisRuntimeException(
                    status=response.status_code,
                    url=url,
                    message=error.get("message"),
                    code=error.get("exception"),
                )
            else:
                raise CmisBaseException(
                    status=response.status_code,
                    url=url,
                    message=error.get("message"),
                    code=error.get("exception"),
                )

        try:
            if response.headers.get("Content-Type").startswith("application/json"):
                return response.json()
            else:
                return response.content.decode("UTF-8")
        except JSONDecodeError:
            if not response.text:
                return None
            raise CmisNoValidResponse(
                status=response.status_code,
                url=url,
                message=response.text,
                code="invalid_response",
            )

    def get_first_result(self, json, return_type):
        if len(json.get("results")) == 0:
            raise GetFirstException()

        return return_type(json.get("results")[0])

    def get_all_results(self, json, return_type):
        results = []
        for item in json.get("results"):
            results.append(return_type(item))
        return results

    def get_all_objects(self, json, return_type):
        objects = []
        for item in json:
            objects.append(return_type(item.get("object")))
        return objects



import re
from collections import namedtuple
from typing import List

from django.core.exceptions import ValidationError

PathElement = namedtuple("PathElement", ["folder_name", "object_type"])
PathElementTemplate = namedtuple("PathElementTemplate", ["folder_name", "required"])

YEAR_PATH_ELEMENT_TEMPLATE = PathElementTemplate(
    folder_name="{{ year }}",
    required=False,
)
MONTH_PATH_ELEMENT_TEMPLATE = PathElementTemplate(
    folder_name="{{ month }}",
    required=False,
)
DAY_PATH_ELEMENT_TEMPLATE = PathElementTemplate(
    folder_name="{{ day }}",
    required=False,
)
ZAAKTYPE_PATH_ELEMENT_TEMPLATE = PathElementTemplate(
    folder_name="{{ zaaktype }}",
    required=True,
)
ZAAK_PATH_ELEMENT_TEMPLATE = PathElementTemplate(
    folder_name="{{ zaak }}",
    required=True,
)


def get_folder_structure(path: str) -> List[PathElement]:
    """Parse a folder path string into path elements.

    A path element example is::

        {
            "folder_name": "Some string",
            "object_type": "cmis:folder"
        }

    :param path: The folder path string
    :return: A `list` of path elements.
    """
    result = []
    folder_pattern = re.compile(r"([^\[]+)(\[[^\]]+\])?")

    for folder in path.strip("/").split("/"):
        if not folder:
            raise ValidationError("Empty path element found. Check for double slashes.")

        folder_match = re.match(folder_pattern, folder).groups()
        if folder_match is None or len(folder_match) != 2:
            raise ValidationError(
                "Invalid path element: %(value)s", params={"value": folder}
            )

        result.append(
            PathElement(
                folder_name=folder_match[0],
                object_type=folder_match[1][1:-1] if folder_match[1] else None,
            )
        )

    return result



from io import BytesIO
from typing import List, Optional, TypeVar, Union
from uuid import UUID

from django.utils import timezone
from django.utils.crypto import constant_time_compare

from cmislib.exceptions import UpdateConflictException

from .models import CMISConfig, Vendor
from .utils import folder as folder_utils
from .utils.exceptions import (
    DocumentConflictException,
    DocumentLockConflictException,
    DocumentNotLockedException,
    FolderDoesNotExistError,
)

# The Document/Folder/Oio/Gebruiksrechten classes used in practice depend on the client
# (different classes exist for the webservice and browser binding)
Document = TypeVar("Document")
Gebruiksrechten = TypeVar("Gebruiksrechten")
Folder = TypeVar("Folder")
ObjectInformatieObject = TypeVar("ObjectInformatieObject")


class CMISClient:

    _main_repo_id = None
    _root_folder_id = None

    document_type = None
    gebruiksrechten_type = None
    oio_type = None
    folder_type = None
    zaakfolder_type = None
    zaaktypefolder_type = None

    def get_other_base_folder_name(self):
        config = CMISConfig.objects.get()
        return config.get_other_base_folder_name()

    def get_zaak_base_folder_name(self):
        config = CMISConfig.objects.get()
        return config.get_zaak_base_folder_name()

    def get_return_type(self, type_name: str) -> type:
        error_message = f"No class {type_name} exists for this client."
        type_name = type_name.lower()
        assert type_name in [
            "zaaktypefolder",
            "zaakfolder",
            "folder",
            "document",
            "gebruiksrechten",
            "oio",
        ], error_message

        if type_name == "folder":
            return self.folder_type
        elif type_name == "document":
            return self.document_type
        elif type_name == "gebruiksrechten":
            return self.gebruiksrechten_type
        elif type_name == "oio":
            return self.oio_type
        elif type_name == "zaaktypefolder":
            return self.zaaktypefolder_type
        elif type_name == "zaakfolder":
            return self.zaakfolder_type

    def get_object_type_id_prefix(self, object_type: str) -> str:
        """Get the prefix for the cmis:objectTypeId.

        Alfresco requires prefixes for create statements of custom objects.
        https://stackoverflow.com/a/28322276/7146757

        :param object_type: str, the type of the object
        :return: str, the prefix (F: or D:)
        """
        if self.vendor.lower() == Vendor.alfresco:
            if object_type in ["zaaktypefolder", "zaakfolder"]:
                return "F:"
            if object_type in ["document", "oio", "gebruiksrechten"]:
                return "D:"

        return ""

    def get_all_versions(self, document: Document) -> List[Document]:
        """Get all versions of a document from the CMS"""
        return document.get_all_versions()

    def get_or_create_folder(
        self, name: str, parent: Folder, properties: dict = None
    ) -> Folder:
        """Get or create a folder 'name/' in the parent folder

        :param name: string, the name of the folder to create
        :param parent: Folder, the parent folder
        :param properties: dict, contains the properties of the folder to create
        :return: Folder, the folder that was created/retrieved
        """
        if properties is None:
            child_type = None
        else:
            child_type = properties.get("cmis:objectTypeId")

        children_folders = parent.get_children_folders(child_type=child_type)
        for folder in children_folders:
            if folder.name == name:
                return folder

        # Create new folder, as it doesn't exist yet
        return self.create_folder(name, parent.objectId, properties)

    def get_folder_by_name(self, name: str, parent: Folder) -> Folder:
        children_folders = parent.get_children_folders(
            child_type=parent.properties.get("cmis:objectTypeId")
        )
        for folder in children_folders:
            if folder.name == name:
                return folder
        raise FolderDoesNotExistError(
            "Folder %(folder_name)s does not exist in %(parent_folder_name)s.",
            params={"folder_name": folder.name, "parent_folder": parent.name},
        )

    def delete_cmis_folders_in_base(self):
        """Delete all the folders in the base folders"""
        config = CMISConfig.get_solo()

        root_folder = self.get_folder(self.root_folder_id)
        for folder_name in set(
            [config.get_zaak_base_folder_name(), config.get_other_base_folder_name()]
        ):
            try:
                folder = self.get_folder_by_name(folder_name, root_folder)
                folder.delete_tree()
            except FolderDoesNotExistError:
                pass

    def update_document(
        self, drc_uuid: str, lock: str, data: dict, content: Optional[BytesIO] = None
    ) -> Document:
        """Update a Document (with the EnkelvoudigInformatieObject properties)

        :param drc_uuid: string, the value of drc:document__uuid
        :param lock: string, value of the lock
        :param data: dict, the new properties of the document
        :param content: BytesIO, the new content of the document
        :return: Document, the updated document
        """
        cmis_doc = self.get_document(drc_uuid)

        if not cmis_doc.isVersionSeriesCheckedOut:
            raise DocumentNotLockedException(
                "Document is not checked out and/or locked."
            )

        if not cmis_doc.lock:
            raise DocumentNotLockedException(
                "Document is not checked out and/or locked."
            )

        correct_lock = constant_time_compare(lock, cmis_doc.lock)

        if not correct_lock:
            raise DocumentLockConflictException("Wrong document lock given.")

        # build up the properties
        current_properties = cmis_doc.properties
        new_properties = self.document_type.build_properties(data, new=False)

        diff_properties = {
            key: value
            for key, value in new_properties.items()
            if current_properties.get(key) != value
        }

        content_filename = data.get("bestandsnaam") or cmis_doc.bestandsnaam

        try:
            if content is not None:
                cmis_doc.update_content(content, content_filename)
            cmis_doc.update_properties(diff_properties)
        except UpdateConflictException as exc:
            # Node locked!
            raise DocumentConflictException from exc

        return cmis_doc

    def update_gebruiksrechten(self, drc_uuid: str, data: dict) -> Gebruiksrechten:
        """Update a gebruiksrechten

        :param drc_uuid: uuid of the gebruiksrechten to update
        :param data: dict of properties to update
        :return: Updated gebruiksrechten
        """

        gebruiksrechten = self.get_content_object(
            drc_uuid=drc_uuid, object_type="gebruiksrechten"
        )

        current_properties = gebruiksrechten.properties
        new_properties = self.gebruiksrechten_type.build_properties(data)

        diff_properties = {
            key: value
            for key, value in new_properties.items()
            if current_properties.get(key) != value
        }

        return gebruiksrechten.update_properties(diff_properties)

    def create_oio(self, data: dict) -> ObjectInformatieObject:
        """Create ObjectInformatieObject which relates a document with a zaak or besluit

        There are a few possible cases.
        1. The oio creates a link to a Besluit:
            A. The besluit is linked to a zaak:
                - If the document is NOT already linked to a zaak, it is moved from the temporary folder to
                    the zaak folder
                - If the document is already linked to a zaak, it will be copied to the new zaak folder
            B. The besluit is not linked to a zaak:
                - If the document is NOT already linked to a zaak, the oio is created in the temporary folder and the
                    document remains in the temporary folder
                - If the document is already linked to a zaak, the document is copied to the temporary folder and
                    the oio is created in the temporary folder
        2. The oio creates a link to a Zaak:
            A. If the document is already related to a zaak, a copy of the document is put in the
                correct zaaktype/zaak folder.
            B. If the document is NOT related to a zaak: the document is moved from the temporary folder
            to the correct zaaktype/zaak folder.

        If the document is linked already to a gebruiksrechten, then the gebruiksrechten object is also moved/copied.

        :param data: dict, the oio details.
        :return: Oio created
        """
        from drc_cmis.client_builder import get_zds_client

        # Get the document
        document_uuid = data.get("informatieobject").split("/")[-1]
        document = self.get_document(drc_uuid=document_uuid)

        if "object" in data:
            data[data["object_type"]] = data.pop("object")

        # Retrieve the zaak and the zaaktype
        if data["object_type"] == "besluit":
            client_besluit = get_zds_client(data["besluit"])
            besluit_data = client_besluit.retrieve("besluit", url=data["besluit"])
            zaak_url = besluit_data.get("zaak")
        else:
            zaak_url = data["zaak"]

        # The oio for the besluit is created in the "Related data" of the temporary folder,
        # since it is not related to a zaak
        if zaak_url is None:
            destination_folder = self.get_or_create_other_folder()
        # The oio is created in the "Related data" folder of the zaak folder
        else:
            client_zaak = get_zds_client(zaak_url)
            zaak_data = client_zaak.retrieve("zaak", url=zaak_url)
            client_zaaktype = get_zds_client(zaak_data["zaaktype"])
            zaaktype_data = client_zaaktype.retrieve(
                "zaaktype", url=zaak_data["zaaktype"]
            )

            # Get or create the destination folder
            destination_folder = self.get_or_create_zaak_folder(
                zaaktype_data, zaak_data
            )

        related_data_folder = self.get_or_create_folder(
            "Related data", destination_folder
        )

        # Check if there are other Oios related to the document
        retrieved_oios = self.query(
            return_type_name="oio",
            lhs=["drc:oio__informatieobject = '%s'"],
            rhs=[data.get("informatieobject")],
        )

        # Check if there are gebruiksrechten related to the document
        related_gebruiksrechten = self.query(
            return_type_name="gebruiksrechten",
            lhs=["drc:gebruiksrechten__informatieobject = '%s'"],
            rhs=[data.get("informatieobject")],
        )

        # Case 1: Already related to a zaak. Copy the document to the destination folder.
        if len(retrieved_oios) > 0:
            self.copy_document(document, destination_folder)
            if len(related_gebruiksrechten) > 0:
                for gebruiksrechten in related_gebruiksrechten:
                    self.copy_gebruiksrechten(gebruiksrechten, related_data_folder)
        # Case 2: Not related to a zaak. Move the document to the destination folder
        else:
            document.move_object(destination_folder)
            if len(related_gebruiksrechten) > 0:
                for gebruiksrechten in related_gebruiksrechten:
                    gebruiksrechten.move_object(related_data_folder)

        # Create the Oio in the "Related data" folder
        return self.create_content_object(
            data=data, object_type="oio", destination_folder=related_data_folder
        )

    def create_gebruiksrechten(self, data: dict) -> Gebruiksrechten:
        """Create gebruiksrechten

        The geburiksrechten is created in the 'Related data' folder in the folder
        of the related document

        :param data: dict, data of the gebruiksrechten
        :return: Gebruiksrechten
        """

        document_uuid = data.get("informatieobject").split("/")[-1]
        document = self.get_document(drc_uuid=document_uuid)

        parent_folder = document.get_parent_folders()[0]
        related_data_folder = self.get_or_create_folder("Related data", parent_folder)

        return self.create_content_object(
            data=data,
            object_type="gebruiksrechten",
            destination_folder=related_data_folder,
        )

    def delete_content_object(self, drc_uuid: Union[str, UUID], object_type: str):
        """Delete the gebruiksrechten/objectinformatieobject with specified uuid

        :param drc_uuid: string or UUID, the value of drc:oio__uuid/drc:gebruiksrechten__uuid
        number gives the cmis:objectId
        :param object_type: string, either "gebruiksrechten" or "oio"
        :return: Either a Gebruiksrechten or ObjectInformatieObject
        """

        content_object = self.get_content_object(drc_uuid, object_type=object_type)
        content_object.delete_object()

    def delete_document(self, drc_uuid: str) -> None:
        """Delete all versions of a document with given uuid

        :param drc_uuid: string, the value of drc:oio__uuid/drc:gebruiksrechten__uuid
        """
        document = self.get_document(drc_uuid=drc_uuid)
        document.delete_object()

    def get_or_create_zaak_folder(self, zaaktype: dict, zaak: dict) -> Folder:
        """Get or create all the folders in the configurable 'zaak' folder path"""
        cmis_config = CMISConfig.get_solo()
        path_elements = folder_utils.get_folder_structure(cmis_config.zaak_folder_path)

        parent_folder = self.get_folder(self.root_folder_id)
        now = timezone.now()

        zaaktype.setdefault(
            "object_type_id",
            f"{self.get_object_type_id_prefix('zaaktypefolder')}drc:zaaktypefolder",
        )
        zaaktype_properties = self.zaaktypefolder_type.build_properties(zaaktype)

        zaak.setdefault(
            "object_type_id",
            f"{self.get_object_type_id_prefix('zaakfolder')}drc:zaakfolder",
        )
        zaak_properties = self.zaakfolder_type.build_properties(zaak)

        ctx = {
            folder_utils.YEAR_PATH_ELEMENT_TEMPLATE.folder_name: (str(now.year), {}),
            folder_utils.MONTH_PATH_ELEMENT_TEMPLATE.folder_name: (str(now.month), {}),
            folder_utils.DAY_PATH_ELEMENT_TEMPLATE.folder_name: (str(now.day), {}),
            folder_utils.ZAAKTYPE_PATH_ELEMENT_TEMPLATE.folder_name: (
                f"zaaktype-{zaaktype.get('omschrijving')}-{zaaktype.get('identificatie')}",
                zaaktype_properties,
            ),
            folder_utils.ZAAK_PATH_ELEMENT_TEMPLATE.folder_name: (
                f"zaak-{zaak['identificatie']}",
                zaak_properties,
            ),
        }

        for pe in path_elements:
            folder_name, props = ctx.get(pe.folder_name, (pe.folder_name, {}))

            parent_folder = self.get_or_create_folder(folder_name, parent_folder, props)

        return parent_folder

    def get_or_create_other_folder(self) -> Folder:
        """Get or create all the folders in the configurable 'other' folder path"""
        cmis_config = CMISConfig.get_solo()
        path_elements = folder_utils.get_folder_structure(cmis_config.other_folder_path)

        parent_folder = self.get_folder(self.root_folder_id)
        now = timezone.now()

        ctx = {
            folder_utils.YEAR_PATH_ELEMENT_TEMPLATE.folder_name: (str(now.year), {}),
            folder_utils.MONTH_PATH_ELEMENT_TEMPLATE.folder_name: (str(now.month), {}),
            folder_utils.DAY_PATH_ELEMENT_TEMPLATE.folder_name: (str(now.day), {}),
        }

        for pe in path_elements:
            folder_name, props = ctx.get(pe.folder_name, (pe.folder_name, {}))

            parent_folder = self.get_or_create_folder(folder_name, parent_folder, props)

        return parent_folder
